// true if user just created the node, and it's not in tree yet
var justCreated = false;

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var tree = null;

var favicons = null;

var utils = null;

var currentNode = null;

var editor;

var strings;

var tagContainer = document.getElementById('prop-tags');
var tagMenu = document.getElementById('menu-tags');

function doInsertLink() {
    var url = prompt("URL:", "http://");
    if (url)
        editor.contentDocument.execCommand("createlink", false, url);
}

function formatBlock () {
	var list = document.getElementById('editor-format');
	console.log("[CP] Yo!" + list.selectedItem.value);
	editor.contentDocument.execCommand('formatblock', false, list.selectedItem.value); 
	list.selectedIndex=0;
}

function clearChildren(node)
{
	while (node.hasChildNodes())
	{
		node.removeChild(node.firstChild);
	}
}

function populateLists (links, property, element) {
	clearChildren(element);
	var list = {};
	for(var i=0; i<links.length; i++)
	{
		var l = links[i];
		list[l[property]] = true;
	}
	list = Object.keys(list);
	list.sort(function (a, b) {
		return a.localeCompare(b);
	});
	
	for(var i=0; i<list.length; i++) 
	{
		var item = document.createElementNS(XUL_NS, "menuitem"); // create a new XUL menuitem
			item.setAttribute("label", list[i]);
		element.appendChild(item);
	}
}

function populateFields(node)
{
	for(var i in fields)
	{
		fields[i].setValue(node[i] || "");
	}

	// prepare variants for author, publisher and type
	var folder = tree.getObject(currentNode.parent);
	var links = folder.links.map(function(val) { return tree.getObject(val); });
	populateLists(links, 'author', document.getElementById('list-author'));
	populateLists(links, 'publisher', document.getElementById('list-publisher'));
	//populateLists(links, 'pubType', document.getElementById('list-pubType'));

	/*for(var i=0; i<node.tags.length; i++)
	{
		if(node.tags[i] == "")
			continue;
		addTag(node.tags[i]);
	}
	updateTags(links);*/
}

function addTag (tag) {
	fields.tags.addTag(tag);
}

function removeTag (value) {
	fields.tags.removeTag(value);
}

function updateTags (links) {
	var tags = {};
	var list = document.getElementById('list-tags');
	clearChildren(list);
	for(var i=0; i<links.length; i++)
	{
		var l = links[i];
		for(var j=0; j<l.tags.length; j++)
			tags[l.tags[j]] = true;
	}
	tags = Object.keys(tags);
	tags.sort(function (a, b) {
		return a.localeCompare(b);
	});
	for(var i=0; i<tags.length; i++)
		if(currentNode.tags.indexOf(tags[i]) == -1)
		{
			var item = document.createElementNS(XUL_NS, "menuitem"); // create a new XUL menuitem
				item.setAttribute("label", tags[i]);
			list.appendChild(item);
		}
}

var fields = {
	'headline': {
		getValue: function() {
			return this.element.value;
		},
		setValue: function(v) {
			this.element.value = v;
		}
	},
	'url': {
		getValue: function() {
			return this.element.value;
		},
		setValue: function(v) {
			this.element.value = v;
		}
	},
	'imageUrl': {
		getValue: function() {
			return this.element.value;
		},
		setValue: function(v) {
			this.element.value = v;
		}
	},
	'description': {
		getValue: function() {
			//console.log("[CP]", this.element.contentDocument.body);
			return this.element.contentWindow.getContent();//this.element.value;
		},
		setValue: function(v) {
			this.element.contentWindow.setContent(v || "");
		}
	},
	'publisher': {
		getValue: function() {
			return this.element.value;
		},
		setValue: function(v) {
			this.element.value = v;
		}
	},
	'author': {
		getValue: function() {
			return this.element.value;
		},
		setValue: function(v) {
			this.element.value = v;
		}
	},
	'pubDate': {
		getValue: function() {
			console.log(this.element.value);
			var t = this.element.value.split('-');
			var a = currentNode.pubDate;
			var to = new Date().getTimezoneOffset();
			var b = new Date( parseInt(t[0], 10), parseInt(t[1], 10) - 1, parseInt(t[2], 10) + (to > 0 ? 1 : 0) ).getTime() - new Date().getTimezoneOffset()*60*1000;
			console.log(a, b, a - b);
			return b;
		},
		setValue: function(v) {
			var d = new Date(v);
			this.element.value = d.getFullYear() + '-' + (d.getMonth() + 1) + '-' + d.getDate();
		}
	},
	'pubType': {
		getValue: function() {
			return this.element.value;
		},
		setValue: function(v) {
			var i = ['default', 'news'].indexOf(v);
			this.element.selectedIndex = i > -1 ? i : 0;
		}
	},
	'tags': {
		tags: [],
		getValue: function() {
			//var result = this.element.value;
			//return result.split(',');
			var tags = document.querySelectorAll('.tag:not(hidden)');
			var result = [];
			for(var i=0; i<tags.length; i++)
				if(tags[i].getAttribute('data-tag') && tags[i].getAttribute('data-tag').trim() !== "")
					result.push(tags[i].getAttribute('data-tag'));
			if(tagMenu.value.trim().length && result.indexOf(tagMenu.value) == -1)
				result.push(tagMenu.value);
			return result;
		},
		hasTag: function(tag) {
			return this.tags.indexOf(tag) > -1;
		},
		addTag: function(tag, silent) {
			tag = tag.toLowerCase();
			if(this.tags.indexOf(tag) == -1) {
				this.tags.push(tag);
				if(!tag || tag.length == 0)
					return;
				var el = document.getElementById('tag-template').cloneNode(true);
				el.removeAttribute('id');
				el.setAttribute('data-tag', tag);
				el.textContent = tag.replace(/ /g, String.fromCharCode(160)); // &nbsp;
				el.hidden = false;
				tagContainer.insertBefore(el, tagMenu);
				if(!silent)
					this.update();
			}
			tagMenu.value = "";
		},
		removeTag: function(tag) {
			var tags = document.querySelectorAll(".tag");
			for(var i=0; i<tags.length; i++)
			{
				if(tags[i].getAttribute('data-tag') == tag)
				{
					tags[i].parentNode.removeChild(tags[i]);
					break;
				}
			}
			this.tags.splice(this.tags.indexOf(tag), 1);
			this.update();
		},
		update: function() {
			var folder = tree.getObject(currentNode.parent);
			var links = folder.links.map(function(val) { return tree.getObject(val); });
			var tags = {};
			var list = document.getElementById('list-tags');
			clearChildren(list);
			for(var i=0; i<links.length; i++)
			{
				var l = links[i];
				for(var j=0; j<l.tags.length; j++)
					tags[l.tags[j]] = true;
			}
			tags = Object.keys(tags);
			tags.sort(function (a, b) {
				return a.localeCompare(b);
			});
			for(var i=0; i<tags.length; i++) {
				if(this.tags.indexOf(tags[i]) == -1)
				{
					var item = document.createElementNS(XUL_NS, "menuitem"); // create a new XUL menuitem
						item.setAttribute("label", tags[i]);
					list.appendChild(item);
				}
			}
		},
		setValue: function(v) {
			for(var i=0; i<v.length; i++)
			{
				if(v[i] == "")
					continue;
				this.addTag(v[i], true);
			}
			this.update();
		}
	},
	'rating': {
		getValue: function() {
			return this.element.selectedIndex;
		},
		setValue: function(v) {
			this.element.selectedIndex = v || 0;
		}
	},
	'twitter': {
		getValue: function() {
			return this.element.value;
		},
		setValue: function(v) {
			this.element.value = v || "";
		}
	}
};

function save()
{
	var data = {};

	for(var i in fields)
	{
		data[i] = fields[i].getValue();
	}
	console.log("[CP]", data);
	if(justCreated)
	{
		for (var i in data)
		{
			currentNode[i] = data[i];
		}
		tree.createNode(currentNode);
		console.log("[CP] Node created", data);
		justCreated = false;
	} else {
		tree.updateNodeInfo(currentNode, data);
	}
	console.log("[CP] Successful save");
}

function paste() 
{
	Components.utils.import('resource://gre/modules/Services.jsm');
	const gClipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                                   .getService(Components.interfaces.nsIClipboardHelper);
	var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
	var source = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                           .getInterface(Components.interfaces.nsIWebNavigation);
	trans.init(source);
	trans.addDataFlavor("text/unicode");
	Services.clipboard.getData(trans, Services.clipboard.kGlobalClipboard);
	var flavor = {};
	var str       = {};
	var strLength = {};
	trans.getAnyTransferData(flavor, str, strLength);
	//console.log(flavor, str, strLength);
	if(str && flavor.value.match(/^text/i))
	{
		const nsSupportsString = Components.Constructor("@mozilla.org/supports-string;1", "nsISupportsString");
		function SupportsString(str) {
		    // Create an instance of the supports-string class
		    var res = nsSupportsString();

		    // Store the JavaScript string that we want to wrap in the new nsISupportsString object
		    res.data = str;
		    return res;
		}
		const nsTransferable = Components.Constructor("@mozilla.org/widget/transferable;1", "nsITransferable");

		// Create a wrapper to construct an nsITransferable instance and set its source to the given window, when necessary
		function Transferable(source) {
		    var res = nsTransferable();
		    if ('init' in res) {
		        // When passed a Window object, find a suitable privacy context for it.
		        if (source instanceof Components.interfaces.nsIDOMWindow)
		            // Note: in Gecko versions >16, you can import the PrivateBrowsingUtils.jsm module
		            // and use PrivateBrowsingUtils.privacyContextFromWindow(sourceWindow) instead
		            source = source.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
		                           .getInterface(Components.interfaces.nsIWebNavigation);

		        res.init(source);
		    }
		    return res;
		}
		var trans = Transferable(window);
		trans.addDataFlavor("text/unicode");
		// We multiply the length of the string by 2, since it's stored in 2-byte UTF-16
		// format internally.
		var pastetext = str.value.QueryInterface(Components.interfaces.nsISupportsString).data;
		//trans.setTransferData("text/unicode", SupportsString(pastetext), pastetext.length * 2);
		pastetext = pastetext.replace(/<script.*\/script>/gi, '').replace(/\<[^\>]+\>/g, '');
		gClipboardHelper.copyString(pastetext);
		document.getElementById('prop-description').contentDocument.execCommand('paste');
	}
}

function twitterInput(event) 
{
	var input = document.getElementById('prop-twitter');
	var val = input.value;
	val = val.replace(/[^a-zA-Z0-9_]/g, '');
	val = val.substr(0, 15);
	if(val.charAt(0) !== '@')
		val = '@' + val;
	input.value = val;
}

function cancel()
{

}

function init()
{
	var args = window.arguments[0];
	tree = args.tree;
	favicons = args.favicons;
	utils = args.utils;
	currentNode = args.node;
	justCreated = args.justCreated;
	editor = document.getElementById('prop-description');
	//additionalInfo = args.info || {};
	//strings = document.getElementById('string-bundle');
	//cacheFields();

	//document.title = strings.getString('title') + " " + getTypeName(args.node) + ": " + args.node.name;

	for(var i in fields)
	{
		fields[i].element = document.getElementById('prop-' + i);
	}

	tagMenu.addEventListener('keypress', function (event) {
		if (event.keyCode == 13)
		{
			addTag(tagMenu.value);
			event.preventDefault();
			event.stopPropagation();
		}
	});

	document.getElementById('menu_cut').addEventListener('command', function() {
		document.getElementById('prop-description').contentDocument.execCommand('cut');
	});

	document.getElementById('menu_copy').addEventListener('command', function() {
		document.getElementById('prop-description').contentDocument.execCommand('copy');
	});

	document.getElementById('menu_paste').addEventListener('command', function() {
		//document.getElementById('prop-description').contentDocument.execCommand('paste');
		paste();
	});

	document.getElementById('prop-twitter').addEventListener('input', twitterInput);

	var ctrlDown = false;
    var ctrlKey = 17, vKey = 86, cKey = 67;
    document.getElementById('prop-description').contentDocument.addEventListener('keypress', function(e) {
    	if(e.keyCode == vKey && e.ctrlKey)
    	{
    		paste();
    		e.preventDefault();
    		return false;
    	}
    });

	function finalize() {
		populateFields(args.node);

		if(justCreated && currentNode.url && currentNode.url.length)
			args.metainfo.getMetaInfo(currentNode.url, function(info) {
				for(var i in info)
				{
					fields[i].setValue(info[i]);
				}
			});
	}

	var editor = document.getElementById('prop-description');
	function wait() {
		if(editor.contentWindow.setContent)
		{
			console.log("FINALIZE");
			finalize();
		} else {
			//console.log("WAIT", editor.contentWindow);
			setTimeout(function() { wait(); }, 1000);
		}
	}
	wait();
	//finalize();
}