var gKeys;
var gOverlays;
var gTree;

var gModifiedOverlays = (function() {
  var overlays = {};

  return {
    get(id) {
      return overlays[id] || null;
    },

    set: function(id, modifiers, key) {
      overlays[id] = {modifiers, key};
      console.log("[HK] Set", overlays);
    },

    disable: function(id) {
      overlays[id] = {disabled: true};
    },

    clear: function(id) {
      delete overlays[id];
    },

    keys: function() {
      return Object.keys(overlays);
    }
   };
})();

function fillHotkeys()
{
	var rows = document.getElementById('hotkeys-rows');
	gKeys = hotkeys.getList();
  gOverlays = hotkeys.getOverlays();
  gOverlays.keys().forEach(function(id) {
    var hotkey = gOverlays.get(id);
    gModifiedOverlays.set(id, hotkey.modifiers, hotkey.key);
  });
  console.log(gOverlays);
  initTree();

  gTree.start();
}

function changeHotkey() {
  gTree.editSelectedRow();
}

function resetHotkey() {
  gModifiedOverlays.clear(gTree.selected);
  gTree.invalidateSelectedRow();
  gTree.stopEditing();
  updateButtons();
}

function updateButtons() {
  var change = document.getElementById('hotkeys-change');
  var reset = document.getElementById('hotkeys-reset');

  change.disabled = reset.disabled = true;

  var id = gTree.selected;
  var overlay = gModifiedOverlays.get(id);

  if (id) {
    change.disabled = false;
  }

  if (overlay) {
    reset.disabled = false;
  }
}

function getKey(id) {
  return gKeys[id] || null;
}

function initTree() {

  var node = document.getElementById("hotkeys-tree");
  var gModifiers = hotkeys.modifiers;
  node.addEventListener("select", updateButtons);
  node.addEventListener("blur", updateButtons);

  // Listen for blur events to determine when to close the panel. We take care
  // of that manually to support Alt+ and Super+ shortcuts on Linux. Some
  // window managers steal focus immediately and would hide the panel.
  addEventListener("blur", function (event) {
    // Don't propagate the event to the tree. We want to continue
    // editing if the edit mode's input field was the active element.
    if (event.originalTarget == node.inputField.inputField) {
      event.stopPropagation();
    }

    // Wait a tick.
    setTimeout(() => {
      if (document.activeElement != node.inputField.inputField) {
        // If the panel is still focused but the active element
        // isn't the edit mode's input field then stop editing.
        gTree.stopEditing();
      }
    });
  }, true);

  addEventListener("keydown", function (event) {
    // Ignore events when not in edit mode.
    if (!node.editingColumn) {
      return;
    }

    event.preventDefault();
    event.stopPropagation();

    var modifiers = gModifiers.fromEvent(event);
    node.inputField.value = gModifiers.toText(modifiers);

    // Bail out if the shortcut isn't compvare yet.
    if (gModifiers.isModifier(event.key)) {
      return;
    }

    var id = gTree.selected;
    var hotkey = gKeys[id];

    var nativeDupe = null;
    var ourDupe = null;
    for(var k in gKeys) {
    	if (gKeys[k].modifiers == modifiers &&
        	gKeys[k].key.toLowerCase() == event.key.toLowerCase()) {
    		// console.log(k, gKeys[k]);
    		nativeDupe = k;
    	}
    	if (gModifiedOverlays.get(k) && gModifiedOverlays.get(k).modifiers == modifiers &&
        	gModifiedOverlays.get(k).key.toLowerCase() == event.key.toLowerCase()) {
    		// console.log(k, gModifiedOverlays.get(k));
    		ourDupe = k;
    	}
    }
    if(nativeDupe) {
    	// console.log("Native");
    	gModifiedOverlays.set(nativeDupe, hotkey.modifiers, hotkey.key);
    }
    if(ourDupe) {
    	// console.log("Our");
    	if(gModifiedOverlays.get(id))
    		gModifiedOverlays.set(ourDupe, gModifiedOverlays.get(id).modifiers, gModifiedOverlays.get(id).key);
    	else
    		gModifiedOverlays.set(ourDupe, hotkey.modifiers, hotkey.key);
    }

    // Check if the shortcut was manually reset to its default value.
    if (hotkey.modifiers == modifiers &&
        hotkey.key.toLowerCase() == event.key.toLowerCase()) {
      // Remove leftovers.
      gModifiedOverlays.clear(id);
    } else {
      // Store the new overlay.
      gModifiedOverlays.set(id, modifiers, event.key);
    }

    node.stopEditing(true);
    updateButtons();
    node.focus();
  }, true);

  gTree = {
    get selected() {
      var row = node.currentIndex;
      var column = node.columns.getLastColumn();
      return node.view.getCellValue(row, column);
    },

    editSelectedRow: function() {
      if (!node.editingColumn) {
        var row = node.currentIndex;
        var column = node.columns.getLastColumn();
        node.startEditing(row, column);
      }
    },

    invalidateSelectedRow: function() {
      var row = node.currentIndex;
      node.treeBoxObject.invalidateRow(row);
    },

    stopEditing: function() {
      node.stopEditing(false);
    },

    start: function() {
      var keys = new Map();
      var ok = Object.keys(gKeys);
      for(var i=0; i<ok.length; i++) {
        keys.set(gKeys[ok[i]].id, gKeys[ok[i]]);
      }

      node.view = gTreeView.create(keys);
    }
  };
}

const gTreeView = (function()
{

	function buildRows(groups)
	{
		var rows = [];

		for (var group of groups)
		{
			var parentIdx = rows.push(group) - 1;

			if (group.open)
			{
				for (key of group.keys)
				rows.push([
					{
						type: "key",
						key,
						parentIdx
					}
				]);
			}
		}
		return rows;
	}

	function create(keys)
	{
    var opened = utils.getSetting('keysOpen', 'string');
    opened = opened ? JSON.parse(opened) : {};
    	var groups = [];
    	for (var [gname, gkeys] of hotkeys.keyGroups.group(keys))
			groups.push(
			{
				type: "group",
				name: gname,
				parentIdx: -1,
				open: opened[gname],
				keys: gkeys
			});

		var rows = buildRows(groups);

		return {
			get rowCount() rows.length,

			isContainer: function(idx)
			{
				return idx >=0 && idx < rows.length && rows[idx].type == "group";
			},

			isEditable: function(idx, column)
			{
				if (column.index == 0 || this.isContainer(idx))
				{
					return false;
				}
        		return true;
			},

			isContainerOpen: function(idx)
			{
				return rows[idx].open;
			},

			getLevel: function(idx)
			{
				return +!this.isContainer(idx);
			},

			getParentIndex: function(idx)
			{
				return rows[idx].parentIdx;
			},

			toggleOpenState: function(idx)
			{
				var row = rows[idx];

        opened[row.name] = !row.open;
        utils.saveSetting('keysOpen', 'string', JSON.stringify(opened));

				var numRows = -row.keys.length;
				if (row.open = !row.open)
				{
					numRows *= -1;
				}

				rows = buildRows(groups);
				this.treebox.rowCountChanged(idx + 1, numRows);
				this.treebox.invalidateRow(idx);
			},

			hasNextSibling: function(idx, after)
			{
				var level = this.getLevel(idx);
				for (var t = after + 1; t < this.rowCount; t++)
				{
					var nextLevel = this.getLevel(t);
					if (nextLevel == level)
					{
						return true;
					}

					if (nextLevel < level)
					{
						return false;
					}
				}
			},

			getCellText: function(idx, column)
			{
				var row = rows[idx];

				if (!row)
				{
					return null;
				}

				if (this.isContainer(idx))
				{
					return column.index ? "" : row.name;
				}

				var key = row.key || row[0].key;
				if (column.index == 0)
				{
					return key.label;
				}

				// Show the new shortcut if overridden.
				var overlay = gModifiedOverlays.get(key.id);
				if (overlay && !overlay.disabled)
				{
					key = overlay;
				}

				return hotkeys.modifiers.toText(key.modifiers) + key.key.toUpperCase();
			},

			getCellValue: function(idx, column)
			{
				if(idx < 0)
					return null;
				if (!this.isContainer(idx))
				{
					return (rows[idx].key || rows[idx][0].key).id;
				}
			},

			getCellProperties: function(idx, column)
			{
				if (this.isContainer(idx))
				{
					return "";
				}

				var props = [];
				var key = rows[idx].key || rows[idx][0].key;

				var overlay = gModifiedOverlays.get(key.id);
				if (column.index && overlay && !overlay.disabled)
				{
					props.push("custom");
				}

				if (overlay && overlay.disabled)
				{
					props.push("disabled");
				}

				return props.join(" ");
			},

			setTree: function(treebox)
			{
				this.treebox = treebox;
			},

			isContainerEmpty: function()
			{
				return false;
			},
			setCellText: function()
			{},
			isSeparator: function()
			{
				return false;
			},
			isSorted()
			{
				return false;
			},
			getImageSrc: function()
			{},
			getRowProperties: function()
			{},
			getColumnProperties: function()
			{},
			cycleHeader: function()
			{}
		};
	}

	return {
		create
	};
})();
