var debugEnabled = false;

// true if user just created the node, and it's not in tree yet
var justCreated = false;

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var tree = null;

var favicons = null;

var utils = null;

var app = null;

var rss = null;

var currentNode = null;

var strings;

var additionalInfo = {};

var pageUrl = null;
var pageName = null;

var stopFlag = false;

var isRSSGroup = false;

var firstShow = true;

var groups = {
	'general':
	{
		title: 'General',
		defaultFocus: 'prop-name',
		index: 0,
		debug: false
	},
	'debug':
	{
		title: 'Debug',
		defaultFocus: 'group-list',
		index: 1,
		debug: true
	}
}

var typeLabels = {
	"root": "Root",
	"panelroot": "Panel Tree",
	"panel": "Panel",
	"group": "Group",
	"page": "Page",
	"linkroot": "Link",
	"link": "Link",
	"curationroot": "Curation Root"
};

var defaultNames = {
	"panel": "New Panel",
	"group": "New Group",
	"page": "New Page",
	"link": "New Link"
}

var linktypes = [
	'web',
	'folder',
	'curation',
	'rss'
];

var pagelinktypes = [
	'web',
	'curation',
	'rss'
];

var props = {};

function cacheFields()
{
	props = {
		name:
		{
			editable: true,
			element: document.getElementById('prop-name'),
			getValue: function()
			{
				var result = this.element.value;
				if(result === '')
					return defaultNames[currentNode.type] || "";
				return result;
			}
		},
		type:
		{
			element: document.getElementById('prop-type'),
			row: document.getElementById('row-type')
		},
		created:
		{
			element: document.getElementById('prop-created')
		},
		tags:
		{
			element: document.getElementById('prop-tags'),
			editable: true,
			getValue: function()
			{
				var result = this.element.value;
				return result.split(',');
			}
		},
		comment:
		{
			editable: true,
			element: document.getElementById('prop-comment'),
			getValue: function()
			{
				var result = this.element.value;
				return result;
			}
		},
		url:
		{
			editable: true,
			element: document.getElementById('prop-url'),
			row: document.getElementById('row-url'),
			getValue: function()
			{
				var result = this.element.value.trim();
				/*if(result == '')
					return utils.getOAPage();*/
				return result;
			}
		},
		page_url:
		{
			editable: true,
			element: document.getElementById('prop-page_url'),
			row: document.getElementById('row-page_url'),
			getValue: function()
			{
				var result = this.element.value;
				return result;
			}
		},
		link:
		{
			editable: true,
			element: document.getElementById('prop-link'),
			row: document.getElementById('row-link'),
			list: document.getElementById('list-link'),
			value: null,
			getValue: function()
			{
				if (this.value)
					return this.value;
				return tree.getLinkRoot().guid;
			}
		},
		linkType:
		{
			editable: false,
			element: document.getElementById('prop-linktype'),
			row: document.getElementById('row-linktype'),
			getValue: function()
			{
				var value = document.getElementById('prop-linktype_create').selectedItem.getAttribute('value');
				if(value == 'web' && props.url.getValue() == '')
					return 'folder';
				if(value == 'folder' && props.url.getValue() !== '')
					return 'web';
				return value;
			}
		},
		page_linktype:
		{
			editable: false,
			element: document.getElementById('prop-page_linktype'),
			row: document.getElementById('row-page_linktype'),
			getValue: function()
			{
				return document.getElementById('prop-page_linktype_create').selectedItem.getAttribute('value');
			}
		},
		guid:
		{
			element: document.getElementById('prop-guid')
		},
		parent:
		{
			element: document.getElementById('prop-parent')
		},
		parenttype:
		{
			element: document.getElementById('prop-parenttype')
		},
		parentname:
		{
			element: document.getElementById('prop-parentname')
		},
		children:
		{
			element: document.getElementById('prop-children')
		},
		publisher:
		{
			editable: true,
			element: document.getElementById('prop-publisher'),
			row: document.getElementById('row-publisher'),
			getValue: function()
			{
				return this.element.value;
			}
		},
		author:
		{
			editable: true,
			element: document.getElementById('prop-author'),
			row: document.getElementById('row-author'),
			getValue: function()
			{
				return this.element.value;
			}
		}
	};
}

function hideRow(prop)
{
	props[prop].row.hidden = true;
}

function showRow(prop)
{
	props[prop].row.hidden = false;
}

function getTypeName(node)
{
	return strings.getString(node.type);
}

function updateLink(id, displayLink)
{
	var link = tree.getObject(id);
	if (!link)
	{
		link = tree.getLinkRoot();
	}
	props.link.value = link.guid;

	displayLink = displayLink || link;
	displayLink = tree.getObject(displayLink);

	var url = pageUrl || displayLink.url;
	var name = pageName || displayLink.name;
	if(displayLink.type == 'linkroot')
		name = pageName;
	console.log("[PR] Name: ", name , " firstShow: ", firstShow, "pageName", pageName);
	if(!pageName && firstShow) 
	{
		console.log("[PR] Resetting name 1");
		if(displayLink.type !== 'linkroot')
			firstShow = false;
		else
			name = '';
	}
	var saveUnder = document.getElementById('comm-saveUnder').checked;
	if(!saveUnder /*&& !(currentNode.type == 'curationpage' && displayLink.linkType != 'curation')*/)
	{
		document.getElementById('prop-page_linktype_create').selectedIndex = pagelinktypes.indexOf(displayLink.linkType);
		url = displayLink.url;
		console.log("[PR] Setting name due to saveUnder", displayLink.name);
		name = displayLink.name;
	}

	document.getElementById('prop-link').setAttribute('label', link.name);
	document.getElementById('prop-page_url').value = url;// displayLink.url;
	document.getElementById('img-page_url').src = favicons.getFavicon(displayLink.domain);
	document.getElementById('prop-page_linktype').value = strings.getString('linktype.' + displayLink.linkType);

	if(currentNode.type == 'curationpage' && saveUnder)
	{
		document.getElementById('prop-page_linktype').value = strings.getString('linktype.curation');
		document.getElementById('prop-page_url').value = '';
	}

	if(currentNode.type == 'curationpage')
	{
		console.log("[PR] Displaylink type: ",displayLink.linkType, displayLink.name);
		var button = document.getElementById('save-button');
		if((curationWizardEnabled || !saveUnder) && displayLink.linkType != 'curation')
		{
			button.disabled = true;
			document.getElementById('addpage-button').disabled = true;
			if(curation.pages.length > 1)
				button.disabled = false;
		} else {
			button.disabled = false;
			document.getElementById('addpage-button').disabled = false;
		}
		if(!pageName && firstShow)
		{
			if(displayLink.type !== 'linkroot')
				firstShow = false;
			else
				name = '';
		}
		if(pageName && firstShow)
		{
			firstShow = false;	
			name = pageName;
		}
		document.getElementById('prop-name').value = name;
	}

	if(currentNode.type == 'page')
	{
		document.getElementById('prop-page_linktype_create').hidden = !saveUnder;
		document.getElementById('prop-page_linktype').hidden = saveUnder;
		props['page_linktype'].editable = saveUnder;
		document.getElementById('prop-name').value = name;
	}

	treeView.focusOnNode(displayLink);

	//var type = pagelinktypes[document.getElementById('prop-page_linktype_create').selectedIndex];
	document.getElementById('prop-page_url').disabled = displayLink.linkType == 'curation' || document.getElementById('prop-page_linktype_create').selectedIndex == 1;
	//document.getElementById('prop-page_url').disabled = type == 'curation';
}

function updateLinkType () {
	if(currentNode.type == 'page' && justCreated && !additionalInfo.createdByDragDrop && tree.getObject(currentNode.link).type == 'linkroot')
	{
		console.log("[PR] Page name : ", pageName);
		var type = pagelinktypes[document.getElementById('prop-page_linktype_create').selectedIndex];
		if(type == 'curation')
		{
			currentNode.link = tree.getCurationFolders().guid;
			updateLink(currentNode.link);
		} else 
		if(type == 'rss')
		{
			currentNode.link = tree.getRSSRoot().guid;
			updateLink(currentNode.link);
		} else {
			currentNode.link = tree.getUnsorted().guid;
			updateLink(currentNode.link);
		}
	}
}

function populateFields(node)
{
	props['name'].element.value = node.name;
	props['comment'].element.value = node.comment;
	props['type'].element.value = getTypeName(node);
	props['created'].element.value = (new Date(node.created)).toLocaleString();
	props['tags'].element.value = node.tags ? node.tags.join(',') : "";

	node.author && (props['author'].element.value = node.author);
	node.publisher && (props['publisher'].element.value = node.publisher);

	var rows = document.querySelectorAll('row');
	var type = node.type == 'linkroot' ? 'link' : node.type;

	for (var i = 0; i < rows.length; i++)
	{
		var row = rows[i];
		var show = row.getAttribute('data-show');
		var hide = row.getAttribute('data-hide');
		row.hidden = false;
		if (show && show != type)
		{
			row.hidden = true;
		}
		if (hide && hide == type)
		{
			row.hidden = true;
		}
		if (show && show == 'page' && type == 'curationpage')
		{
			row.hidden = false;
		}
	}

	if(justCreated && node.type !== 'curationpage')
	{
		document.getElementById('prop-page_linktype_create').hidden = false;
		document.getElementById('prop-page_linktype').hidden = true;
		props['page_linktype'].editable = true;

		document.getElementById('prop-linktype_create').hidden = false;
		document.getElementById('prop-linktype').hidden = true;
		props['linkType'].editable = true;
	}

	if (node.type == 'link' || node.type == 'linkroot')
	{
		console.log("LT: " + node.linkType);
		document.getElementById('prop-linktype').value = strings.getString('linktype.' + node.linkType);
		document.getElementById('prop-linktype_create').selectedIndex = linktypes.indexOf(node.linkType);
		document.getElementById('prop-page_linktype_create').selectedIndex = linktypes.indexOf(node.linkType);
		/*if(node.type == 'linkroot')
			props['url'].element.disabled = true;*/
		/*if(node.linkType == 'curation')
			props['url'].row.hidden = true;*/
		props['url'].element.value = node.url.trim();
		document.getElementById('img-url').src = favicons.getFavicon(node.domain);

		if(justCreated && additionalInfo.createdByDragDrop)
		{
			rss.detectRSS(node.url, null, function(result, url, guid) {
				if(result)
				{
					//node.name = result;
					//document.getElementById('prop-name').value = result;
					document.getElementById('prop-linktype_create').selectedIndex = linktypes.indexOf('rss');
					document.getElementById('img-url').src = favicons.getFavicon('rss');
				}
			});	
		}

	}

	if(node.type == 'group' && additionalInfo.createdByDragDrop)
	{
		rss.detectRSS(additionalInfo.createdByDragDrop, null, function(result, url, guid) {
			if(result)
			{
				//additionalInfo.pagename = result;
				isRSSGroup = true;
			}
		});	
	}

	if (node.type == 'page')
	{
		//console.log('[PR] URL', additionalInfo.createdByDragDrop, 'LINK', node.link);
		if(justCreated && additionalInfo.createdByDragDrop)
		{
			node.link = tree.getUnsorted().guid;
			document.getElementById('comm-saveUnder').checked = true;
			document.getElementById('prop-page_url').value = additionalInfo.createdByDragDrop;
			pageUrl = additionalInfo.createdByDragDrop;
			pageName = node.name;
			rss.detectRSS(pageUrl, null, function(result, url, guid) {
				if(result)
				{
					document.getElementById('prop-page_linktype_create').selectedIndex = 2; // not via "indexOf" because we removed "web folder" for pages
					node.link = tree.getRSSRoot().guid;
					//document.getElementById('prop-name').value = result;
					//pageName = node.name = result;
					updateLink(tree.getRSSRoot().guid);
					document.getElementById('img-page_url').src = favicons.getFavicon('rss');
				}
			});
		}
		if(justCreated && !additionalInfo.createdByDragDrop && !additionalInfo.createdByLinkDrop && tree.getObject(node.link).type == 'linkroot')
		{
			pageName = '';
			document.getElementById('comm-saveUnder').checked = true;
		}
		document.getElementById("linkTree").view = treeView;
		treeView.prepareTree(tree.getLinkRoot()); 
		treeView.treeBox.invalidate();
		updateLink(tree.getObject(node.link));
		document.getElementById('comm-saveUnder').hidden = !justCreated;
	}

	if (node.type == 'curationpage')
	{
		console.log("[PR] Node.link ", node.link);	
		if(justCreated && !node.link)
		{
			node.link = tree.getCurationFolders();
		}
		document.getElementById('comm-saveUnder').hidden = true;
		document.getElementById("linkTree").view = treeView;
		treeView.prepareTree(tree.getLinkRoot());
		treeView.treeBox.invalidate();

		pageName = node.name;
		if(justCreated && !additionalInfo.createdByDragDrop && node.link == tree.getCurationFolders().guid)
		{
			pageName = '';
		}
		updateLink(tree.getObject(node.link));
		updateSaveUnder();
	}


	// debug
	props['guid'].element.value = node.guid;
	props['parent'].element.value = node.parent;
	var parent = tree.getObject(node.parent);
	props['parentname'].element.value = parent ? parent.name : "";
	props['parenttype'].element.value = parent ? parent.type : null;
	for (var i = 0; i < node.children.length; i++)
	{
		var item = document.createElementNS(XUL_NS, "listitem");
		var cell = document.createElementNS(XUL_NS, "listcell");
		var child = tree.getObject(node.children[i]);
		cell.setAttribute('label', child.name);
		item.appendChild(cell);

		cell = document.createElementNS(XUL_NS, "listcell");
		cell.setAttribute('label', child.type);
		item.appendChild(cell);

		cell = document.createElementNS(XUL_NS, "listcell");
		cell.setAttribute('label', child.guid);
		item.appendChild(cell);

		props['children'].element.appendChild(item);
	}
}

function selectGroup(group)
{
	if (typeof(group) != 'string')
	{
		var item = document.getElementById('group-list').selectedItem;
		console.log("[PR] ", item);
		var group = item.getAttribute('data-type');
	}
	else
	{
		document.getElementById('group-list').selectedItem = document.getElementById('group-' + group);
	}
	console.log("[PR] Selecting group " + group, groups[group]);
	document.getElementById('properties-deck').selectedIndex = groups[group].index;
}

function appendGroup(type)
{
	var list = document.getElementById('group-list');
	var item = document.createElementNS(XUL_NS, "listitem");
	console.log("[PR] Appending " + type + " (" + groups[type].title + ")");
	item.setAttribute('id', 'group-' + type);
	item.setAttribute('label', strings.getString(type));
	item.setAttribute('data-type', type);
	list.addEventListener('select', selectGroup);
	list.appendChild(item);
}

function removeGroup(type)
{
	var list = document.getElementById('group-list');
	var item = document.getElementById('group-' + type);
	list.removeChild(item);
}

function switchDebugMode()
{
	var list = document.getElementById('group-list');
	if (debugEnabled)
	{
		debugEnabled = false;
		console.log("[PR] Hacker has disappeared");
	}
	else
	{
		debugEnabled = true;
		console.log("[PR] Advanced hacking technique!");
	}
	for (var i in groups)
	{
		if (groups.hasOwnProperty(i))
			if (groups[i].debug == debugEnabled)
				appendGroup(i);
			else
				removeGroup(i);
	}
	list.selectedIndex = 0;
}

function clearChildren(node)
{
	while (node.hasChildNodes())
	{
		node.removeChild(node.firstChild);
	}
}

function filterGroups(type)
{
	var list = document.getElementById('group-list');
	clearChildren(list);
	appendGroup(type);
}

function treeSelect(index)
{
	var t = document.getElementById('linkTree');
	//var item = document.getElementById('list-link-item');

	var link = tree.getObject(treeView.visibleData[index].guid);
	console.log("[PR] ", t.currentIndex, link.name);
	document.getElementById('list-link').hidePopup();
	var displayLink = link;
	var saveUnder = document.getElementById('comm-saveUnder').checked;
	if(justCreated && currentNode.type == 'page')
	{
		if(currentNode.link !== link.guid && saveUnder)
		{
			//document.getElementById('comm-saveUnder').checked = true;
			//displayLink = currentNode.link;
		}
	}
	updateLink(link, displayLink);	
}

function treeMouseUp(event)
{
	if (event.button != 0)
		return;
	var row = {},
		col = {},
		child = {};
	treeView.treeBox.getCellAt(event.clientX, event.clientY, row, col, child);
	var idx = row.value;
	debug && console.log("[TV] Mouse Up! ", row, col);
	if (idx > -1 && idx < treeView.visibleData.length)
	{
		popupShowing = false;
		setTimeout(function()
		{
			if (!treeView.nodeToggle)
			{
				treeSelect(idx);
			}
			treeView.nodeToggle = false;
		}, 100);
	}
}

function markPageUrl()
{
	pageUrl = props.page_url.getValue();
}

function markPageName()
{
	pageName = props.name.getValue();
}

function updateSaveUnder(event)
{
	var saveUnder = document.getElementById('comm-saveUnder').checked;
	var link = tree.getObject(props.link.getValue());
	var displayLink = tree.getObject(currentNode.link);
	var url = props.page_url.getValue();
	if(currentNode.type == 'curationpage')
	{
		if(saveUnder)
		{
			document.getElementById('prop-page_linktype_create').selectedIndex = 2;
			document.getElementById('prop-page_linktype_create').disabled = true;
		} else {
			document.getElementById('prop-page_linktype_create').disabled = true;
		}
	}
	console.log("[PR] displayLink", displayLink);
	updateLink(link, saveUnder ? displayLink : link);
}

function save()
{
	if(stopFlag) 
	{
		window.close();
		return;
	}
	stopFlag = true;

	if (currentNode.type == 'root' || currentNode.type == 'panelroot' || currentNode.type == 'linkroot')
	{
		window.close();
		return;
	}

	if(curationWizardEnabled)
	{
		curationSave();
		window.close();
		return;
	}

	var data = {};
	for (var i in props)
		if (props.hasOwnProperty(i))
		{
			if (props[i].editable && currentNode.hasOwnProperty(i))
			{
				data[i] = props[i].getValue();
			}
		}

		// for pages we also must save link info
		// 
	if(currentNode.type == 'page')
	{
		var saveUnder = document.getElementById('comm-saveUnder').checked;

		var link = /*saveUnder && currentNode.link !== tree.getLinkRoot().guid ? tree.getObject(currentNode.link) : */tree.getObject(props.link.getValue());
		var new_url = props.page_url.getValue();

		var urlChanged = link.url !== new_url;
		var isLinkroot = link.type == 'linkroot';
		var dragDrop = additionalInfo.createdByDragDrop;
		var linkChanged = currentNode.link !== link.guid;

		console.log("[P] urlChanged=" + urlChanged + ", isLinkroot=" + isLinkroot + ", dragDrop=" + dragDrop + ", linkChanged=" + linkChanged);

		/*if(justCreated && saveUnder && link.guid !== props.link.getValue())
		{
			tree.moveSubtree(link, tree.getObject(props.link.getValue()));
			data.link = link.guid;
		}*/

		if(currentNode.link == tree.getUnsorted().guid && new_url.trim() == "" && saveUnder) 
		{
			new_url = utils.getOAPage();
			link = tree.getLinkRoot().guid;
			data['link'] = link;
			saveUnder = false;
			isLinkroot = false;
		}

		// update link
		if(urlChanged && !isLinkroot && !saveUnder)
		{
			console.log("[P] Updating link " + link.name + " -> " + data.name + " " + new_url);
			tree.updateNodeInfo(link, { url: new_url });
		}

		// create new link
		if(saveUnder || urlChanged && (justCreated && !dragDrop || !justCreated && isLinkroot))
		{
			console.log("[P] Creating new link " + data.name + " " + new_url);
			var linkInfo = {
					name: data.name,
					url: new_url,
					parent: null
				};
			if(saveUnder)
			{
				console.log("Save under checked, parent is " + link.name);
				linkInfo.parent = link.guid;
				var type = document.getElementById('prop-page_linktype_create').selectedItem.getAttribute('value');
				linkInfo.linkType = type;
			}
			var new_link = tree.createLink(linkInfo);
			data['link'] = new_link.guid;
			linkChanged = true;
		} else
		//if(!linkChanged)
			tree.updateNodeInfo(link, { name: data.name });
	}

	if(currentNode.type == 'curationpage')
	{
		var saveUnder = document.getElementById('comm-saveUnder').checked;
		var link = tree.getObject(props.link.getValue());

		if(saveUnder)
		{
			 var newlink = tree.createLink({
			 	name: data.name,
			 	linkType: 'curation',
			 	url: 'about:blank',
			 	parent: link.guid
			 });
			 data['link'] = newlink.guid;
		} else {
			data['link'] = link.guid;
			tree.updateNodeInfo(link, { name: data.name });
		}
	}

	if (justCreated)
	{
		if(currentNode.type == 'group')
		{
			currentNode.pagename = additionalInfo.pagename || (additionalInfo.link ? tree.getObject(additionalInfo.link).name : "New Page");
			currentNode.url = additionalInfo.url || additionalInfo.createdByDragDrop;	
			currentNode.link = additionalInfo.link || (currentNode.url ? null : tree.getLinkRoot().guid);
			if(isRSSGroup)
			{
				var link = tree.createLink({
					name: additionalInfo.pagename || "New Page",
					linkType: 'rss',
					url: additionalInfo.createdByDragDrop,
					parent: tree.getRSSRoot().guid
				});
				currentNode.link = link.guid;
			}


			//console.log("[PR] AI", additionalInfo);
		}

		if(currentNode.type == 'link')
		{
			var p = currentNode.parent;
			if(p == tree.getUnsorted().guid)
			{
				if(currentNode.linkType == 'rss')
					currentNode.parent = tree.getRSSRoot().guid;
				if(currentNode.linkType == 'curation')
					currentNode.parent = tree.getCurationFolders().guid;
			}
		}

		for (var i in data)
		{
			if (data.hasOwnProperty(i))
				currentNode[i] = data[i];
		}
		tree.createNode(currentNode);

		justCreated = false;

	}
	else
	{
		tree.updateNodeInfo(currentNode, data);

		// reset page
		if(currentNode.type == 'page' && (linkChanged || urlChanged && !justCreated))
		{
			console.log("[P] Resetting page");
			tree.resetPage(currentNode, true);
		}
	}
	window.close();
}

function cancel()
{
	/*if (justCreated && additionalInfo.createdByDragDrop && currentNode.type == 'page')
	{
		tree.removeNode(currentNode.link);
	}*/
	window.close();
}

function exportCurationFile()
{
	app.exportCurationFile(currentNode);
}

function init()
{
	var args = window.arguments[0];
	tree = args.tree;
	app = args.app;
	favicons = args.favicons;
	utils = args.utils;
	rss = args.rss;
	currentNode = args.node;
	justCreated = args.justCreated;
	additionalInfo = args.info || {};
	strings = document.getElementById('properties-string-bundle');
	cacheFields();
	filterGroups('general');
	selectGroup('general');

	document.getElementById('list-link').addEventListener('popupshowing', function(event) 
	{
		var tree = document.getElementById('linkTree');
		if(tree.currentIndex > -1)
			tree.treeBoxObject.ensureRowIsVisible(tree.currentIndex);
	});

	if(additionalInfo.curationWizard)
	{
		initCurationWizard();
	} else {
		document.title = strings.getString('title') + " " + getTypeName(args.node) + ": " + args.node.name;
		populateFields(args.node);
	}

	var group = groups['general'];
	if (group.defaultFocus)
	{
		console.log("[PR] Focusing " + group.defaultFocus);
		document.getElementById(group.defaultFocus).focus();
	}

	document.getElementById('prop-name').addEventListener('keyup', function() {
		pageName = document.getElementById('prop-name').value;
	});

	document.addEventListener('keypress', function(event)
	{
		// ctrl + d combination
		if ((event.charCode == 68 || event.charCode == 100) && event.ctrlKey)
		{
			switchDebugMode();
		}
	});
}
