
var icons = {
	"root": "root",
	"panelroot": "folder",
	"linkroot": "folder",
	"panel": "panel",
	"group": "folder",
	"page": "page",
	"link": "link"
};

// shorthand functions
var o = function o(guid)
{
	return tree.getObject(guid);
}

var treeView = {
	childData:
	{

	},

	/**
	 * Array of rows in treeview.<br>
	 * format of visibleData elements: <br>
	 * {<br>
	 *   name: string<br>
	 *   guid: string<br>
	 *   container: bool<br>
	 *   level: int<br>
	 *   open: bool,<br>
	 *   properties: array of strings<br>
	 *   type: string<br>
	 * }
	 * @type {Array}
	 */
	visibleData: [
	{
		name: "",
		guid: "adasda",
		container: false,
		type: 'link',
		open: false,
		level: 0
	}],

	/**
	 * The nsITreeBoxObject interface contains information about the size and layout of a tree.
	 * @type {nsITreeBoxObject}
	 */
	treeBox: null,
	selection: null,
	rootLevel: 0,
	selectionReact: true,
	nodeToggle: false,

	/**
	 * Makes object suitable for visibleData array
	 * @param  {BaseObject} node      tree node to represent in tree view
	 * @param  {int} rootLevel root level of currently displayed subtree. 0 for root, 2 for panels, 3 for groups etc.
	 * @param  {boolean} open      is node open
	 * @return {object}           see visibleData structure
	 */
	makeVisibleNode: function(node, rootLevel, open)
	{
		var icon = "chrome://store.tabsmoz/content/tree/img/" + icons[node.type] + ".png";
		if (node.type == 'link')
			icon = favicons.getFavicon(node.domain);
		if (node.type == 'page')
		{
			var link = o(node.link);
			if (link)
				icon = favicons.getFavicon(link.domain);
		}
		return {
			name: node.name,
			guid: node.guid,
			container: (node.children.length == 0 || node.type == 'page') ? false : true,
			open: node.open,
			level: node.level - rootLevel,
			type: node.type,
			icon: icon,
			isRoot: node.level == rootLevel,
			properties: []
		};
	},

	/**
	 * populate visibleData from main tree
	 * @param  {BaseObject} root root node of subtree
	 * @param {boolean} open is tree opened
	 */
	prepareTree: function(root, open)
	{
		//this.visibleData = [];
		var data = []; //this.visibleData;
		var rootLevel = root.level;
		var self = this;

		function traverse(node)
		{
			data.push(self.makeVisibleNode(node, rootLevel, true));
			//console.log("[TV] " + node.name + ": " + node.open);
			if (data[data.length - 1].open && node.type != 'page')
				for (var i = 0; i < node.children.length; i++)
					traverse(o(node.children[i]));
		}
		traverse(root);
		this.rootLevel = rootLevel;
		//console.log('[TV] ', this.visibleData);
		var l = this.visibleData.length;
		this.visibleData = data;
		this.treeBox.rowCountChanged(0, data.length - l);
	},

	get rowCount()
	{
		return this.visibleData.length;
	},
	setTree: function(treeBox)
	{
		this.treeBox = treeBox;
	},
	getCellText: function(idx, column)
	{
		if (idx < 0 || idx >= this.visibleData.length) return "";
		return this.visibleData[idx].name;
	},
	isContainer: function(idx)
	{
		if (idx < 0 || idx >= this.visibleData.length) return false;
		return this.visibleData[idx].container;
	},
	isContainerOpen: function(idx)
	{
		if (idx < 0 || idx >= this.visibleData.length) return false;
		return this.visibleData[idx].open;
	},
	isContainerEmpty: function(idx)
	{
		return false;
	},
	isSeparator: function(idx)
	{
		return false;
	},
	isSorted: function()
	{
		return false;
	},
	isEditable: function(idx, column)
	{
		return false;
	},

	getParentIndex: function(idx)
	{
		if (idx < 0 || idx >= this.visibleData.length) return -1;
		if (this.visibleData[idx].isRoot) return -1;
		for (var t = idx - 1; t >= 0; t--)
		{
			if (this.visibleData[t].level == this.visibleData[idx].level - 1) return t;
		}
		return 0;
	},
	getLevel: function(idx)
	{
		if (idx < 0 || idx >= this.visibleData.length) return -1;
		return this.visibleData[idx].level;
	},
	hasNextSibling: function(idx, after)
	{
		var thisLevel = this.getLevel(idx);
		for (var t = after + 1; t < this.visibleData.length; t++)
		{
			var nextLevel = this.getLevel(t);
			if (nextLevel == thisLevel) return true;
			if (nextLevel < thisLevel) break;
		}
		return false;
	},
	toggleOpenState: function(idx)
	{
		console.log("[TV] Open/Close!");
		this.nodeToggle = true;
		this.selectionReact = false;
		setTimeout(function() {
			treeView.selectionReact = true;	
			treeView.nodeToggle = false;
		}, 300);
		var item = this.visibleData[idx];

		if (!item.container) return;

		var node = o(this.visibleData[idx].guid);
		if (item.open)
		{
			item.open = false;
			//node.open = false;
			var thisLevel = this.getLevel(idx);
			var deletecount = 0;
			for (var t = idx + 1; t < this.visibleData.length; t++)
			{
				if (this.getLevel(t) > thisLevel) deletecount++;
				else break;
			}
			if (deletecount)
			{
				this.visibleData.splice(idx + 1, deletecount);
				this.treeBox.rowCountChanged(idx + 1, -deletecount);
			}
		}
		else
		{
			item.open = true;
			//node.open = true;
			var rowsAdded = 0;
			var self = this;

			function traverse(node)
			{
				for (var i = 0; i < node.children.length; i++)
				{
					var c = o(node.children[i]);
					rowsAdded++;
					self.visibleData.splice(idx + rowsAdded, 0, self.makeVisibleNode(c, self.rootLevel, c.open));
					if (c.open)
						traverse(c);
				}
			}
			traverse(node);
			this.treeBox.rowCountChanged(idx + 1, rowsAdded);
		}
		this.treeBox.invalidateRow(idx);

		//tree.toggleOpen(item.guid, item.open);
	},

	focusOnNode: function(node)
	{
		var self = this;
		function openNode(guid)
		{
			for(var i=0; i<self.visibleData.length; i++)
				if(self.visibleData[i].guid == guid && !self.visibleData[i].open)
				{
					self.toggleOpenState(i);
					return;
				}
		}
		var p = o(node.parent);
		var parents = [];
		while(p.parent)
		{
			parents.push(p.guid);
			p = o(p.parent);
		}
		for(var i=0; i<parents.length; i++)
			openNode(parents[i]);
		for(var i=0; i<this.visibleData.length; i++)
			if(this.visibleData[i].guid == node.guid)
			{
				console.log("[PR] Focusing on node");
				this.selectionReact = false;
				document.getElementById('linkTree').view.selection.select(i);
				setTimeout(function() {
					treeView.selectionReact = true;	
					treeView.nodeToggle = false;
				}, 300);
			}
	},

	getImageSrc: function(idx, column)
	{
		if(column.id == 'name')
			return this.visibleData[idx].icon;
	},
	getProgressMode: function(idx, column) {},
	getCellValue: function(idx, column) {
		if(column.id == 'check') {
			var s = selection[this.visibleData[idx].guid];
			return s === undefined ? false : s;
		}
	},
	cycleHeader: function(col, elem) {},
	selectionChanged: function() {},
	cycleCell: function(idx, column) {},
	performAction: function(action) {},
	performActionOnCell: function(action, index, column) {},
	getRowProperties: function(idx)
	{
		if (this.visibleData[idx] && this.visibleData[idx].properties)
			return this.visibleData[idx].properties.join(' ');
		return "";
	},
	getCellProperties: function(idx, column, prop)
	{
		/*if(this.visibleData[idx].properties)
      return this.visibleData[idx].properties.join(' ');*/
		return "";
	},
	getColumnProperties: function(column, element, prop)
	{
		return "";
	},
};