var tree,
	xul,
	utils,
	hotkeys,
	app;

function init()
{
	var args = window.arguments[0];
	tree = args.tree;
	xul = args.xul;
	utils = args.utils;
	hotkeys = args.hotkeys;
	app = args.app;

	var defaultFile = utils.getSetting("defaultFile", "string");
	document.getElementById('prop-file').value = (defaultFile) || "";

	var onStartup = utils.getSetting("onStartup", "string") || "normal";
	switch (onStartup)
	{
		case "normal":
			document.getElementById('prop-startup').selectedIndex = 0;
			break;
		case "last":
			document.getElementById('prop-startup').selectedIndex = 1;
			break;
	}

	var colorTheme = utils.getSetting('colorTheme', "int");
	updateColorTheme(colorTheme);

	var groupTabPosition = utils.getSetting('groupTabPosition', "string") || "right";
	setRadio('prop-groupTabPosition', groupTabPosition);

	var subpageTabPosition = utils.getSetting('subpageTabPosition', "string") || "bottom";
	setRadio('prop-subpageTabPosition', subpageTabPosition);

	var toolbarPosition = utils.getSetting('toolbarPosition', "string") || 'left';
	setRadio('prop-toolbarPosition', toolbarPosition);

	function loadBool(name) {
		var value = utils.getSetting(name, "bool") || false;
		document.getElementById('prop-' + name).checked = value;
	}

	function loadInt(name, defValue) {
		var def = defValue || 0;
		var value = utils.getSetting(name, "int") || def;
		document.getElementById('prop-' + name).value = value;
	}

	loadBool("saveOnExit");
	loadInt("saveInterval");
	loadInt("groupDelay");
	loadInt("hoverDelay");
	loadInt("hoverLinkDelay");
	loadInt("maxSubpageTabs", 15);
	loadBool("showPropertiesOnCreate");
	loadBool("warnLastPage");
	loadBool("warnDeleteGroup");
	loadBool("warnDeletePanel");
	loadBool("hidePageTabs");
	loadBool("hideGroupTabs");
	loadBool("showCommentTooltip");
	loadInt("historySize");
	loadBool("showToolbar");
	loadInt("rssItemsCount", 0);

	updateSubpageTabPosition();

	fillHotkeys();
}

function updateColorTheme(index) {
	var boxes = document.querySelectorAll('.colorbox');
	for(var i=0; i<boxes.length; i++)
		boxes[i].removeAttribute('active');
	boxes[index + 1].setAttribute('active', true);
}

function colorboxClick(event) {
	updateColorTheme(+event.target.getAttribute('index'));
}

function setRadio(id, value)
{
	var group = document.getElementById(id);
	group.selectedItem = group.querySelector("radio[value='" + value + "']");
}

function updateSubpageTabPosition() {
	var subpageTabPosition = document.getElementById('prop-subpageTabPosition');
	var groupHidden = document.getElementById('prop-hideGroupTabs');
	var pageHidden = document.getElementById('prop-hidePageTabs');
	var subpageTop = document.querySelector('#prop-subpageTabPosition radio[value="top"]');

	if(groupHidden.checked && pageHidden.checked) {
		//subpageTop.disabled = false;
		subpageTabPosition.disabled = false;
	} else {
		/*subpageTop.disabled = true;
		if(subpageTabPosition.selectedIndex == 0)
			subpageTabPosition.selectedIndex ++;*/
		subpageTabPosition.disabled = true;
	}
}

function chooseFile()
{
	var nsIFilePicker = Components.interfaces.nsIFilePicker;
	var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
	fp.init(window, "Select a File", nsIFilePicker.modeSave);
	fp.appendFilter("JSON files", "*.json");
	fp.defaultExtension = ".json";
	var result = fp.show();
	if (result != nsIFilePicker.returnCancel)
	{
		var thefile = fp.file;
		var data = tree.serializeJSON();
		utils.saveSetting("defaultFile", "string", (thefile.path));
		document.getElementById('prop-file').value = thefile.path;
		utils.asyncWriteLocal(thefile, data, function()
		{
			if (!thefile.path.match(/\.json$/i))
			{
				console.log("[SETT] Renaming " + thefile.leafName + " to " + thefile.leafName + ".json", thefile.parent);
				thefile.moveTo(null, thefile.leafName + ".json");
				utils.saveSetting("defaultFile", "string", (thefile.path));
			}
		});
	}
}

function saveAutosave()
{
	utils.saveSetting("autosave", "bool", document.getElementById('prop-autosave').checked);
}

function saveNumber(name)
{
	var value = parseInt(document.getElementById("prop-" + name).value);
	if (!isNaN(value))
	{
		utils.saveSetting(name, "int", value);
	}
}

function saveBool(name)
{
	utils.saveSetting(name, "bool", document.getElementById('prop-' + name).checked);
}

function save()
{
	utils.saveSetting("defaultFile", "string", document.getElementById('prop-file').value);
	utils.saveSetting("onStartup", "string", document.getElementById('prop-startup').selectedItem.value);
	utils.saveSetting("groupTabPosition", "string", document.getElementById('prop-groupTabPosition').selectedItem.value);
	utils.saveSetting("toolbarPosition", "string", document.getElementById('prop-toolbarPosition').selectedItem.value)
	saveBool("saveOnExit");
	saveBool("warnLastPage");
	saveBool("warnDeleteGroup");
	saveBool("warnDeletePanel");
	saveBool("hidePageTabs");
	saveBool("hideGroupTabs");
	saveBool("showPropertiesOnCreate");
	saveBool("showCommentTooltip");
	saveBool("showToolbar");

	var subpageTabPosition = document.getElementById('prop-subpageTabPosition');
	if(subpageTabPosition.disabled)
		utils.saveSetting("subpageTabPosition", "string", "bottom");
	else
		utils.saveSetting("subpageTabPosition", "string", subpageTabPosition.selectedItem.value);

	var lastInterval = utils.getSetting("saveInterval", "int");
	var newInterval = parseInt(document.getElementById("prop-saveInterval").value);
	if (!isNaN(newInterval) && lastInterval !== newInterval)
	{
		utils.saveSetting("saveInterval", "int", newInterval < 0 ? 0 : newInterval);
		app.setSaveInterval(newInterval);
	}

	saveNumber("groupDelay");
	saveNumber("hoverDelay");
	saveNumber("hoverLinkDelay");
	saveNumber("maxSubpageTabs");
	saveNumber("historySize");
	saveNumber("rssItemsCount");

	var colorbox = document.querySelector('.colorbox[active]');
	utils.saveSetting("colorTheme", "int", parseInt(colorbox.getAttribute('index'), 10));
	console.log(parseInt(colorbox.getAttribute('index'), 10));

	utils.emitEvent(utils.uiModifiedEvent, {});

	gOverlays.keys().forEach(function(id) {
		gOverlays.clear(id);
	});
	gModifiedOverlays.keys().forEach(function(id) {
		var hotkey = gModifiedOverlays.get(id);
		gOverlays.set(id, hotkey.modifiers, hotkey.key);
	});
	hotkeys.save();
}

function changeGroup () {
	var idx = document.getElementById('group-list').selectedIndex;
	document.getElementById('properties-deck').selectedIndex = idx;
}