
var tree,
	xul,
	utils,
	app,
	setUpdater;

function init()
{
	var args = window.arguments[0];
	tree = args.tree;
	utils = args.utils;
	app = args.app;
	setUpdater = args.setUpdater;

	setUpdater(checkTabs);

	fillList();
}

function checkTabs(data)
{
	switch(data.type)
	{
		case 'delete':
			if(data.target.type == 'subpage') {
				var row = document.querySelector('row[data-guid="' + data.target.guid + '"]');
				row.parentNode.removeChild(row);
			}
		break;
		case 'add':
			if(data.target.type == 'subpage') {
				createRow(data.target);
			}
		break;
		case 'update':
			var label = document.querySelector('label[data-guid="' + data.target.guid + '"]');
			if(label)
				label.setAttribute('value', data.target.name);
		break;
	}
	updateInfo();
}

function createRow(node)
{
	var rows = document.getElementById("rowContainer");
	var template = document.getElementById("rowTemplate");
	var item = template.cloneNode(true);
	var labels = item.getElementsByTagName('label');
	console.log("[TM] ", labels.length);
	var path = [node];
	var parent = tree.getObject(node.parent);
	while (parent && parent.type !== 'root' && parent.type !== 'panelroot' && parent.type !== 'linkroot')
	{
		if (parent)
			path.unshift(parent);
		parent = tree.getObject(parent.parent);
	}
	for(var i=0; i<labels.length; i++) {
		if(path[i].type == 'subpage')
			labels[i].setAttribute("value", path[i].local ? path[i].name : path[i].label);
		else
			labels[i].setAttribute("value", path[i].name);
		labels[i].setAttribute("data-guid", path[i].guid);
	}
	item.querySelector('.oaff-cb').setAttribute('data-guid', node.guid);
	item.setAttribute('data-guid', node.guid);
	item.hidden = false;
	rows.appendChild(item);
}

function fillList()
{
	var subpages = tree.getArrayOfType('subpage');

	var rows = document.getElementById("rowContainer");

	while(rows.children.length > 2)
		rows.removeChild(rows.children[2]);

	var list = [];

	function traverse(node)
	{
		if(node.type == 'subpage')
		{
			list.push(node);
		} else {
			if(node.children && node.children.length)
				for(var i=0; i<node.children.length; i++)
					traverse(tree.getObject(node.children[i]));
		}
	}

	traverse(tree.getSelected('panel'), []);

	for(var i=0; i<list.length; i++) {
		createRow(list[i]);
	}

	updateInfo();
}

function updateInfo()
{
	var total = document.querySelectorAll('.oaff-cb');
	var checked = 0;
	for(var i=1; i<total.length; i++) {
		if(total[i].checked)
			checked++;
	}
	document.getElementById('info').textContent = "Total: " + (total.length - 1) + ", " + checked + " selected";
}

function closeTabs()
{
	var cb = document.querySelectorAll('.oaff-cb');
	for(var i=1; i<cb.length; i++) 
	{
		if(cb[i].checked) 
		{
			var guid = cb[i].getAttribute('data-guid');
			tree.removeNode(guid);
		}
	}
	window.close();	
}

function selectAll()
{
	var check = document.getElementById('cbSelectAll').checked;
	var cb = document.querySelectorAll('.oaff-cb');
	for(var i=1; i<cb.length; i++) {
		cb[i].checked = check;
	}
	updateInfo();
}

function exit()
{
	setUpdater(null);
	window.close();
}