
var tree,
	utils,
	favicons,
	clipboard;

var mainTree,
	currentNode;

var showColumns = {
	'panelroot':  ['name', 'type', 'created', 'tags', 'comment'],
	'panel':      ['name', 'type', 'created', 'tags', 'comment'],
	'group':      ['name', 'type', 'created', 'tags', 'comment'],
	'page':       ['name', 'linkType', 'created', 'tags', 'comment'],
	'link':       ['name', 'linkType', 'created', 'tags', 'comment'],
	'linkroot':   ['name', 'linkType', 'created', 'tags', 'comment'],
	'curation':   ['headline', 'pubDate', 'publisher', 'author', 'tags', 'rating'],
	'curationroot': ['name', 'type', 'created', 'tags', 'comment'],
	'curationfile': ['name', 'type', 'created', 'tags', 'comment'],
	'curationgroup': ['name', 'type', 'created', 'tags', 'comment'],
	'curationpage': ['name', 'type', 'created', 'tags', 'comment'],
	'search': ['name', 'type', 'created', 'path'],
	'rss': ['name']
}

var columns = [
	'name',
	'type',
	'linkType',
	'created',
	'headline',
	'pubDate',
	'publisher',
	'author',
	'tags',
	'rating',
	'comment',
	'path'
];

var linkTypes = {
	'web': 'Web Link',
	'folder': 'Web Folder',
	'curation': 'Curation Folder'
};

var ratingStrings = [
	"*",
	"**",
	"***",
	"****",
	"*****"
];

var sortMap = {};

var treeStrings = document.getElementById("treeview-strings");
var propStrings = document.getElementById("properties-strings");

// state variables
var selectedNode = null;
var hoveredNode = null;
var currentIndex = -1;
var popupShowing = false;
var nodeToggle = false;
var canEdit = -1;
var doubleClicked = false;
var lastSelected = null;

var rssViewType = 0;
var currentRSSData = null;
var rssUpdateFlag = false;

var nodePreviewed = null;

var debug = false;

function $(id)
{
	return document.getElementById(id);
}

function o(guid)
{
	return tree.getObject(guid);
}

var treeView = {
	visibleData: [],

	makeVisibleNode: function(node)
	{
		var colSet = currentNode.linkType == 'curation' ? showColumns['curation'] : showColumns[currentNode.type];
		var result = {};
		for(var i=0; i<colSet.length; i++) {
			var col = colSet[i];
			if(col == 'name')
			{
				if(node.type == 'rsslink')
					result.name = node.title;
				else if(node.type == 'curationlink')
					result.name = node.headline;
				else
					result.name = node.name;
			} else
			if(col == 'rating')
			{
				result.rating = ratingStrings[node.rating];
			} else
			if(col == 'pubDate')
			{
				result[col] = (new Date(node.pubDate)).toLocaleDateString();
				result.date = node.pubDate;
			} else
			if(col == 'created')
			{
				result[col] = (new Date(node.created)).toLocaleDateString();
			} else
			if(col == 'tags')
			{
				result[col] = node.tags.join(',');
			} else
			if(col == 'linkType')
			{
				result[col] = propStrings.getString('linktype.' + node.linkType);
			} else
			if(col == 'type')
			{
				if(node.linkType)
					result[col] = propStrings.getString('linktype.' + node.linkType);
				else
					result[col] = propStrings.getString(node.type);
			} else
			if(col == 'path')
			{
				result[col] = node.getFullPath().join('/');
			} else
				result[col] = node[col];
		}
		result.guid = node.guid;
		return result;
	},

	prepareTree: function()
	{
		console.log("Preparing tree");
		if(!currentNode || currentNode.linkType == 'rss')
			return;	
		var data = [];
		var guids = currentNode.linkType == 'curation' || currentNode.linkType == 'rss' ? currentNode.links : currentNode.children;
		for(var i=0; i<guids.length; i++) {
			data.push(this.makeVisibleNode(tree.getObject(guids[i])));
		}
		var l = this.visibleData.length;
		this.visibleData = data;
		this.treebox.rowCountChanged(0, data.length - l);
		this.treebox.invalidate();
		// show/hide columns
		var colSet = (currentNode.linkType == 'curation' ? showColumns['curation'] : showColumns[currentNode.type]);
		if(currentNode.linkType == 'rss')
			colSet = showColumns['rss'];
		var cols = document.querySelectorAll('treecol');
		for(var i=0; i<cols.length; i++) {
			var field = cols[i].getAttribute('data-field');
			var hide = colSet.indexOf(field) < 0;
			cols[i].hidden = hide;
			if(hide)
				cols[i].setAttribute('ignoreincolumnpicker', true);
			else
				cols[i].removeAttribute('ignoreincolumnpicker');
		}
		//sort();
		/*if(currentNode.linkType == 'curation')
		{
			mainTree.setAttribute('seltype', 'multiple');
		} else {
			mainTree.setAttribute('seltype', 'single');
		}*/
	},

	isChild: function(guid) {
		for(var i=0; i<this.visibleData.length; i++)
		{
			if(this.visibleData[i].guid == guid)
				return i;
		}
		return -1;
	},

	get rowCount()
	{
		return this.visibleData.length;
	},

	drop: function (idx, orientation, dataTransfer) {

		function copymove(src, dst)
		{
			src = tree.getObject(src);
			if(!clipboard.canManipulate([src]))
				return;
			dst = dst || node;
			if (dataTransfer.dropEffect == 'move')
			{
				tree.moveSubtree(src, dst);
			}
			if (dataTransfer.dropEffect == 'copy')
			{
				tree.copySubtree(src, dst);
			}
		}

		var node;
		if(idx < 0 || idx >= this.visibleData.length)
			node = currentNode;
		else
			node = tree.getObject(this.visibleData[idx].guid);

		if(node.type == 'curationlink')
			node = tree.getObject(node.parent);

		var data = utils.extractURL(dataTransfer);

		// copy rss link into curation folder. Very special case
		if(data.nodelist && data.nodetype == 'rsslink' && node.linkType == 'curation')
		{
			for(var i=0; i<data.nodelist.length; i++)
			{
				var r = tree.getObject(data.nodelist[i]);
				var c = tree.createCurationLink({
					parent: node.guid,
					url: r.url,
					headline: r.title,
					author: r.author,
					description: r.content,
					pubDate: r.pubDate,
					imageUrl: r.img,
					tags: r.tags
				});
				console.log(r.tags, c.tags);
			}
			return;
		}
		if(data.type == 'rsslink')
		{
			var r = tree.getObject(data.node);
			var c = tree.createCurationLink({
				parent: node.guid,
				url: r.url,
				headline: r.title,
				author: r.author,
				description: r.content,
				pubDate: r.pubDate,
				imageUrl: r.img,
				tags: r.tags
			});
			return;
		}
		if(data.nodelist)
		{
			var nodes = data.nodelist;
			if(clipboard.compatible(node.type, node, data.nodetype))
			{
				var nodes = data.nodelist;
				for(var i=0; i<nodes.length; i++)
				{
					copymove(nodes[i]);
				}
				return;
			}
		}
		if(node.type == 'curationlink' && data.url)
		{
			var link = tree.createCurationLink({
				parent: node.parent,
				url: data.url,
				title: data.name
			}, true);
			window.parent.Tabsmoz.showProperties(link, true);
		} else
		if(node.linkType == 'curation' && !data.guid)
		{
			if(data.type == 'curationlink')
			{
				if(node.links.indexOf(data.node) < 0)
				{
					copymove(data.node, node);
				}
				return;
			} else
			if(data.url)
			{
				var url = data.url || "",
					name = data.name || "";
				var link = tree.createCurationLink({
					parent: node.guid,
					url: url,
					headline: name
				}, true);
				window.parent.Tabsmoz.showProperties(link, true);
			}
		} else
		if(node.type == 'link' || node.type == 'linkroot')
		{
			if(data.url && data.link == undefined)
			{
				var link = tree.createLink(
				{
					name: data.name,
					url: data.url,
					parent: node.guid
				}, true);
				window.parent.Tabsmoz.showProperties(link, true);
			}
			if (data.link !== undefined && data.link !== node.guid)
			{
				if (tree.getObject(data.link).type == 'linkroot')
					return;
				copymove(data.link);
			}
		} else
		if (data.node || data.link)
		{
			var src = tree.getObject(data.node || data.link	);
			if (clipboard.compatible.call(
			{
				type: src.type
			}, node.type))
			{
				copymove(src, node);
			}
		}
	},

	cycleHeader: function(col)
	{
		sort(col);
	},

	setTree: function(treebox){ this.treebox = treebox; },
    isContainer: function(row){ return false; },
    isSeparator: function(row){ return false; },
    isSorted: function(){ return true; },
    isEditable: function (row, col) {
    	return col.element.hasAttribute('editable');	
    },
    getLevel: function(row){ return 0; },
    getImageSrc: function(row,col) { return null; },
    getRowProperties: function(row,props) {},
    getCellProperties: function(row,col,props) {},
    getColumnProperties: function(colid,col,props) {},

    getCellText: function(row, col) {
    	return this.visibleData[row][columns[col.index]];
    },
    setCellText: function (row, col, value) {
    	var node = tree.getObject(this.visibleData[row].guid);
    	var property = col.element.getAttribute('data-field');
    	if(property == 'tags')
    		value = value.split(',');
    	var info = {};
    	info[property] = value;
    	tree.updateNodeInfo(node, info);
    }
};

function sort (column) {
	var order = mainTree.getAttribute("sortDirection") == "ascending" ? 1 : -1;
	var columnName;
	//if the column is passed and it's already sorted by that column, reverse sort
	if (column) {
		columnName = column.id;
		if (mainTree.getAttribute("sortResource") == columnName) {
			order *= -1;
		}
	} else {
		if(sortMap[currentNode.id])
		{
			columnName = sortMap[currentNode.id].columnName;
			column = { element: document.getElementById(columnName) };
			order = sortMap[currentNode.id].order;
		} else {
			var cols = document.querySelectorAll('treecol');
			for(var i=0; i<cols.length; i++)
				cols.removeAttribute('sortDirection');
			return;
		}
	}
	sortMap[currentNode.id] = {
		column: columnName,
		order: order
	};
	var cols = mainTree.getElementsByTagName("treecol");
	for (var i = 0; i < cols.length; i++) {
		cols[i].removeAttribute("sortDirection");
	}
	mainTree.setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
	column.element.setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
	mainTree.setAttribute("sortResource", columnName);
	console.log("SORT " + order, columnName);

	var property = columnName.substr(4, columnName.length);
	if(property == 'name' || property == 'headline' || property == 'author' || property == 'publisher')
		treeView.visibleData.sort(function sortTree (a, b) {
			return b[property].toLowerCase().localeCompare(a[property].toLowerCase()) * order;
		});
	else if(property == 'pubDate')
		treeView.visibleData.sort(function sortTree (a, b) {
			if(a['date'] < b['date']) return -1 * order;
			if(a['date'] > b['date']) return 1 * order;
			return 0;
		});
	else
		treeView.visibleData.sort(function sortTree (a, b) {
			if(a[property] < b[property]) return -1 * order;
			if(a[property] > b[property]) return 1 * order;
			return 0;
		});
	treeView.treebox.invalidate();
}

function getSelection() 
{
	var selection = [];
	var start = {};
	var end = {};
	var numRanges = mainTree.view.selection.getRangeCount();
	for(var i=0; i<numRanges; i++)
	{
		mainTree.view.selection.getRangeAt(i, start, end);
		for(var j=start.value; j<=end.value; j++)
			selection.push(tree.getObject(treeView.visibleData[j].guid));
	}
	return selection;
}

function showTreePopup(event)
{
	// just curation links for now. Oh.
	var multiSelect = mainTree.view.selection.count > 1;
	var idx = currentIndex;
	var selection = getSelection();
	if(idx < 0)
	{
		selectedNode = null;
		var t = currentNode.type;
		if(t == 'linkroot')
			t = 'link';
		if(currentNode.linkType == 'curation')
			t = 'curationlink';
		if(t !== 'page' && t !== 'curationpage')
			$('oaff-tree-menu-new').label = treeStrings.getString('new.' + t);
		$('oaff-tree-menu-cut').disabled = true;
		$('oaff-tree-menu-copy').disabled = true;
		$('oaff-tree-menu-paste').disabled = !clipboard.compatible(currentNode.type, currentNode);
		$('oaff-tree-menu-properties').disabled = true;
		$('oaff-tree-menu-delete').disabled = true;
		if(t == 'link')
		{
			$('oaff-tree-menu-new').hidden = true;
			$('oaff-tree-menu-new-link').hidden = false;
		} else {
			$('oaff-tree-menu-new').hidden = false;
			$('oaff-tree-menu-new-link').hidden = true;
		}
	}
	else //if(selection.length == 1)
	{
		var node = o(treeView.visibleData[idx].guid);
		var disableCopyPaste = !clipboard.canManipulate(selection);
		var disableDelete = !clipboard.canDelete(selection);
		$('oaff-tree-menu-cut').disabled = disableCopyPaste;
		$('oaff-tree-menu-copy').disabled = disableCopyPaste;
		$('oaff-tree-menu-paste').disabled = !clipboard.compatible(node.type, node);
		$('oaff-tree-menu-properties').disabled = node.type == 'rsslink';
		$('oaff-tree-menu-delete').disabled = disableDelete;

		var t = node.type;
		if(t == 'linkroot')
			t = 'link';
		if(t == 'link')
		{
			$('oaff-tree-menu-new').hidden = true;
			$('oaff-tree-menu-new-link').hidden = false;
		} else {
			$('oaff-tree-menu-new').hidden = false;
			$('oaff-tree-menu-new-link').hidden = true;
		}
		if(t == 'rsslink')
		{
			$('oaff-tree-menu-new').disabled = true;
			$('oaff-tree-menu-delete').disabled = true;
		}
		if (t !== 'page' && t !== 'curationpage')
		{
			$('oaff-tree-menu-new').label = treeStrings.getString('new.' + t);
			$('oaff-tree-menu-newsep').hidden = false;
		} else {
			$('oaff-tree-menu-new').hidden = true;
			$('oaff-tree-menu-newsep').hidden = true;
		}
		$('oaff-tree-menu-delete').label = treeStrings.getString('delete.' + t);
		$('oaff-tree-menu-cut').label = treeStrings.getString('cut.' + t);
		$('oaff-tree-menu-copy').label = treeStrings.getString('copy.' + t);
		$('oaff-tree-menu-paste').label = treeStrings.getString('paste.' + (clipboard.node == null ? "null" : clipboard.node.type));

		selectedNode = node;
	} /*else {
		selectedNode = null;

		var disableCopyPaste = !clipboard.canManipulate(selection);
		$('oaff-tree-menu-cut').disabled = disableCopyPaste;
		$('oaff-tree-menu-copy').disabled = disableCopyPaste;
		$('oaff-tree-menu-paste').disabled = true;
		$('oaff-tree-menu-properties').disabled = true;
		$('oaff-tree-menu-delete').disabled = false;

		$('oaff-tree-menu-new').hidden = true;
		$('oaff-tree-menu-new-link').hidden = false;

		$('oaff-tree-menu-delete').label = treeStrings.getString('delete.nodes');
		$('oaff-tree-menu-cut').label = treeStrings.getString('cut.nodes');
		$('oaff-tree-menu-copy').label = treeStrings.getString('copy.nodes');
		$('oaff-tree-menu-paste').label = treeStrings.getString('paste.null');
	}*/

}

function newNode(event, linkType)
{
	switch(currentNode.type)
	{
		case 'linkroot':
		case 'link':
			if(currentNode.linkType == 'curation')
			{
				var link = tree.createCurationLink({
					parent: currentNode.guid
				}, true);
				window.parent.Tabsmoz.showProperties(link, true);
			} else {
				var linkType = linkType || 'folder';
				var link = tree.createLink({
					parent: currentNode.guid,
					linkType: linkType
				}, true);
				window.parent.Tabsmoz.showProperties(link, true);
			}
		break;

		case 'panelroot':
			var panel = tree.createPanel({
				parent: currentNode.guid
			}, true);
			window.parent.Tabsmoz.showProperties(panel, true);
		break;

		case 'panel':
			var group = tree.createGroup({
				parent: currentNode.guid
			}, true);
			window.parent.Tabsmoz.showProperties(group, true);
		break;

		case 'group':
			var page = tree.createPage({
				parent: currentNode.guid
			}, true);
			window.parent.Tabsmoz.showProperties(page, true);
		break;

		case 'curationroot':
			var file = tree.createCurationFile({
				parent: currentNode.guid
			}, true);
			window.parent.Tabsmoz.showProperties(file, true);
		break;

		case 'curationfile':
			var group = tree.createCurationGroup({
				parent: currentNode.guid
			}, true);
			window.parent.Tabsmoz.showProperties(group, true);
		break;

		case 'curationgroup':
			var page = tree.createCurationPage({
				parent: currentNode.guid
			}, true);
			window.parent.Tabsmoz.showProperties(page, true);
		break;
	}
}

function cutNode (event) 
{
	var selection = getSelection();
	clipboard.set(selection, 'cut');
}

function copyNode (event) 
{
	var selection = getSelection();
	clipboard.set(selection, 'copy');
}

function pasteNode (event) 
{
	var data = clipboard.data;
	if (clipboard.action == 'cut')
	{
		for(var i=0; i<data.length; i++)
			tree.moveSubtree(data[i], selectedNode || currentNode);
		clipboard.clear();
	}
	if (clipboard.action == 'copy')
	{
		for(var i=0; i<data.length; i++)
			tree.copySubtree(data[i], selectedNode || currentNode);
	}
}

function removeNode (event) 
{
	var selection = getSelection();
	for(var i=0; i<selection.length; i++)
		tree.removeNode(selection[i]);	
	mainTree.view.selection.clearSelection();
}

function showProperties(event)
{
	window.parent.Tabsmoz.showProperties(selectedNode);
}

function hideTreePopup(event)
{
	popupShowing = false;
}

function treeBlur(event)
{
	hoveredNode = null;
	//treeView.highlightNode(-1);
}

function treeMouseUp(event)
{
	if (event.button != 0)
		return;
	var row = {},
		col = {},
		child = {};
	treeView.treebox.getCellAt(event.clientX, event.clientY, row, col, child);
	var idx = row.value;
	if (idx > -1 && idx < treeView.visibleData.length)
	{
		setTimeout(function() {
			lastSelected = treeView.visibleData[idx].guid;
		}, 600);
		setTimeout(function() {
			if(currentNode && treeView.visibleData[idx].guid == lastSelected)
			{
				console.log("Edit");
				mainTree.startEditing(row.value, col.value);
			}
		}, 500);
		popupShowing = false;
		setTimeout(function()
		{
			console.log("Showing preview ", idx);
			nodeToggle = false;
			if(!currentNode)
				return;
			
			showNodePreview(treeView.visibleData[idx].guid);
			if(currentNode.linkType == 'curation')
			{
				currentNode.lastChild = treeView.visibleData[idx].guid;
			}
		}, 100);
		
	} else {
		document.getElementById('mainTree').view.selection.clearSelection();
		lastSelected = null;
	}
}

function treeDblClick(event) {
	var row = {},
		col = {},
		child = {};
	doubleClicked = true;
	event.preventDefault();

	treeView.treebox.getCellAt(event.clientX, event.clientY, row, col, child);
	var idx = row.value;
	if (idx > -1 && idx < treeView.visibleData.length)
	{
		var node = tree.getObject(treeView.visibleData[idx].guid);
		if(node.type == 'curationlink' || node.type == 'rsslink')
		{
			if(currentNode.type == 'search')
			{
				tree.getObject(node.parent).lastChild = node.guid;
				window.parent.Tabsmoz.openSubpage({
					//url: node.url
					node: node.parent,
					local: true,
					name: tree.getObject(node.parent).name
				});
			} else {
				window.parent.Tabsmoz.openSubpage({
					url: node.url
				});
			}

		} else
		if(node.type == 'link' || node.type == 'linkroot')
		{
			window.parent.Tabsmoz.openSubpage({
				node: node.guid,
				local: node.local,
				url: node.url,
				name: node.name
			});
		} else {
			if(window.parent.Tabsmoz.getManagementMode() == 'local')
			{
				tree.openLinkInPage(tree.getSelected('page'), node.guid);
			} else {
				window.parent.Tabsmoz.selectNode(node.guid);
			}
		}
	}
}

function treeSelect(event)
{
	var idx = document.getElementById('mainTree').currentIndex;
	if(idx > -1 && idx < treeView.visibleData.length)
	{
		showNodePreview(treeView.visibleData[idx].guid);
	}
}

/**
 * Handles start of dragging node, sets dataTransfer object with data according to node properties
 * @param  {Event} event
 */
function treeDragStart(event)
{
	var idx = document.getElementById('mainTree').currentIndex;
	var node = tree.getObject(treeView.visibleData[idx].guid);
	var selection = getSelection();
	if (selection.length > 1)
	{
		event.dataTransfer.setData('text/x-tabsmoz-nodelist', selection.map(function (val) {
			return val.guid;
		}));
		event.dataTransfer.setData('text/x-tabsmoz-nodetype', selection[0].type);
	} else {
		if (node.type == 'link' || node.type == 'linkroot')
		{
			event.dataTransfer.setData('text/x-moz-url-data', node.url);
			event.dataTransfer.setData('text/x-moz-url-desc', node.name);
			event.dataTransfer.setData("text/uri-list", node.url);
			event.dataTransfer.setData('text/x-tabsmoz-link', node.guid);
			debug && console.log("[LV] Dragging " + node.name);
		}
		if (node.type == "page" || node.type == "group" || node.type == "panel" || node.type == 'curationgroup' || node.type == 'curationpage')
		{
			if (node.type !== "panel")
				event.dataTransfer.setData('text/x-tabsmoz-tab', node.guid);
			event.dataTransfer.setData('text/x-tabsmoz-node', node.guid);
			debug && console.log("[LV] Dragging " + node.name);
		}
		if (node.type == 'curationlink')
		{
			event.dataTransfer.setData('text/x-moz-url-data', node.url);
			event.dataTransfer.setData('text/x-moz-url-desc', node.headline);
			event.dataTransfer.setData("text/uri-list", node.url);
			event.dataTransfer.setData('text/x-tabsmoz-node', node.guid);
			event.dataTransfer.setData('text/x-tabsmoz-type', 'curationlink');
			debug && console.log("[LV] Dragging " + node.headline);
		}
	}
}

function treeDragOver(event)
{
	var row = {},
		col = {},
		child = {};
	treeView.treebox.getCellAt(event.clientX, event.clientY, row, col, child);
	if (row.value > -1 && row.value < treeView.visibleData.length)
	{
		//mainTree.view.selection.select(row.value);
		var node = treeView.visibleData[row.value];
		var data = utils.extractURL(event.dataTransfer);
		var over = tree.getObject(data.node);
		var under = tree.getObject(node.guid);
		if(data.node && over.guid !== under.guid && over.type == under.type && over.type !== 'link' && over.type !== 'linkroot')
		{
			tree.swapNodes(over, under);
		}
	}
	event.preventDefault();
}

function updateListView (data) {
	var node = data.target,
		type = data.type;
	switch(type)
	{
		case 'add':
			nodeAdded(node);
			break;
		case 'update':
			nodeUpdated(node);
			break;
		case 'delete':
			nodeDeleted(node);
			break;
	}
}

function nodeAdded (node) {
	if(node.parent == currentNode.guid && node.type !== 'rsslink') {
		//console.log("[LV] nodeAdded");
		treeView.prepareTree();
		showNodePreview(node);
		//sort();
	}
	if(node.type == 'rsslink' && node.parent == currentNode.guid)
	{
		showRSSFeed();
	}
}

function nodeUpdated (node, data) {
	var i = treeView.isChild(node.guid);
	if(i > -1)
	{
		//console.log("[LV] nodeUpdated");
		treeView.prepareTree();
		showNodePreview(node);
	}
	if(node.guid == currentNode.guid)
	{
		treeView.prepareTree();
	}
	if(node.type == 'rsslink' && node.parent == currentNode.guid)
	{
		showRSSFeed();
	}
}

function nodeDeleted(node) {
	if(node.type == 'rsslink' && node.parent == currentNode.guid)
	{
		showRSSFeed();
	}
	if(treeView.isChild(node.guid) > -1)
	{
		//console.log("[LV] nodeDeleted");
		treeView.prepareTree();
		if(mainTree.selectedIndex >= 0)
		{
			var node = tree.getObject(treeView.visibleData[mainTree.selectedIndex].guid);
			showNodePreview(node);
		} else 
			showNodePreview(null);
		console.log("Sorting due to node deleting");
		sort();
	}
	if(node.guid == currentNode.guid)
	{
		document.getElementById('common-view').hidden = false;
		document.getElementById('rss-view').hidden = true;
	}
}

function readMore()
{
	var url = document.getElementById('description-more').getAttribute('data-url');
	if(url)
	{
		window.parent.Tabsmoz.openSubpage({
			url: url
		});
	}
}

function showNode(node)
{
	mainTree.stopEditing(false);
	if(!node)
	{
		currentNode = null;
		clearRSS();
		document.getElementById('common-view').hidden = false;
		document.getElementById('rss-view').hidden = true;
		document.getElementById('preview').hidden = true;
		var cols = document.querySelectorAll('treecol');
		for(var i=0; i<cols.length; i++) {
			cols[i].hidden = true;
		}
		return;
	}
	
	tree = tree || window.parent.Tabsmoz.getTree();
	utils = utils || window.parent.Tabsmoz.getUtils();
	favicons = favicons || window.parent.Tabsmoz.getFavicons();
	clipboard = clipboard || window.parent.Tabsmoz.clipboard;

	if(!currentNode || node.guid != currentNode.guid) {
		currentNode = node;

		if(node.linkType == 'rss')
		{
			//console.log("[LV] Showing rss");
			showRSSFeed();
			unblockRefreshButton();
			if(node.links.length == 0)
			{
				//console.log("[LV] Refreshing");
				refreshRSS();
			}
			document.getElementById('common-view').hidden = true;
			document.getElementById('rss-view').hidden = false;
		} else {
			document.getElementById('common-view').hidden = false;
			document.getElementById('rss-view').hidden = true;
			document.getElementById('preview').hidden = node.linkType != 'curation';
			try {
				treeView.prepareTree();
				if(treeView.visibleData.length > 0)
				{
					var l = currentNode.getLastChild();
					var idx = -1;
					for(var i=0; i<treeView.visibleData.length; i++)
					{
						if(treeView.visibleData[i].guid == l)
						{
							idx = i;
							break;
						}
					}
					if(idx > -1)
					{
						mainTree.view.selection.select(idx);
						showNodePreview(treeView.visibleData[idx].guid);
						mainTree.treeBoxObject.ensureRowIsVisible(idx);
					}
					console.log("Sorting to show node");
					sort();
				}
				else
				{
					mainTree.view.selection.clearSelection();
					showNodePreview(null);
				}
			} catch(e) {
				console.log("[LV] Failed to show tree");
			};
		}
	}
}

function switchRSSType()
{
	var newType = document.getElementById('rss-view-type').selectedIndex;
	if(newType == rssViewType)
		return;
	rssViewType = newType;
	utils.saveSetting("rssviewtype", "int", rssViewType);
	showRSSFeed(currentRSSData);
}

function refreshRSS()
{
	tree.refreshRSSLink(currentNode);
	document.getElementById('rss-refresh-button').disabled = true;
	document.getElementById('rss-refresh-button').label = "Refreshing...";
	setTimeout(function() {
		unblockRefreshButton(null, true);
	}, 5000);
}

function clearRSS()
{
	document.getElementById('rss-feed-title').value = "";
	document.getElementById('rss-feed-title').setAttribute('data-url', "");

	document.getElementById('rss-frame').contentWindow.clearFeed();
}

function showRSSFeed()
{
	function realShowRSS()
	{
		if(!currentNode || !currentNode.links)
			return;
		document.getElementById('rss-view-type').selectedIndex = utils.getSetting('rssviewtype', 'int') || 0;
		var data = {
			title: currentNode.feedTitle,
			url: currentNode.feedUrl,
			posts: []
		};
		for(var i=0; i<currentNode.links.length; i++)
		{
			var entry = tree.getObject(currentNode.links[i]);
			data.posts.push(entry);
		}
		currentRSSData = data;
		
		document.getElementById('rss-feed-title').value = data.title;
		document.getElementById('rss-feed-title').setAttribute('data-url', data.url);

		document.getElementById('rss-frame').contentWindow.showFeed(data, rssViewType);
	}

	if(!rssUpdateFlag)
	{
		rssUpdateFlag = true;
		setTimeout(function() {
			realShowRSS();
			rssUpdateFlag = false;
		}, 500);
	}
}

function openSubpage(url, rssId) {
	window.parent.Tabsmoz.openSubpage({ url: url });
	if(rssId)
	{
		var link = tree.getObject(rssId);
		tree.updateNodeInfo(link, {read: true});
	}
	//window.parent.gBrowser.addTab(url);
}

function unblockRefreshButton(id, force)
{
	if(currentNode && currentNode.guid == id || force)
	{
		document.getElementById('rss-refresh-button').disabled = false;
		document.getElementById('rss-refresh-button').label = "Refresh";
	}
}

function showNodePreview (node) {
	nodePreviewed = node;
	if(!node)
	{
		var frame = document.getElementById('description-preview');
		frame.contentWindow.setContent("");
		document.getElementById('description-more').disabled = true;
		return;
	}
	node = tree.getObject(node);
	if(node.type != 'curationlink')
		return;
	var frame = document.getElementById('description-preview');
	frame.contentWindow.setContent("<h1>" + node.headline + "</h1><p>" + (node.description || "") + "</p>");
	document.getElementById('description-more').disabled = false;
	document.getElementById('description-more').setAttribute("data-url", node.url);
}

function onKeyPress(event)
{
	console.log(event.which);
	if (event.ctrlKey && event.which !== 0 && String.fromCharCode(event.which).toLowerCase() == 'a') 
	{
    	mainTree.view.selection.selectAll();	
    }
}

function reset()
{
	//console.log("[LV] Reset")
	currentNode = null;
}

function init () {
	mainTree = document.getElementById('mainTree');
	mainTree.view = treeView;

	mainTree.addEventListener('keypress', onKeyPress);

	document.getElementById('oaff-tree-menu-new-link').label = treeStrings.getString('new.link');

	//console.log("[RSS]", document.getElementById('rss-refresh-button'));

	document.getElementById('rss-feed-title').addEventListener('click', function(event)
	{
		var url = event.target.getAttribute('data-url');
		if(url)
			openSubpage(url);
	});	

	// right click handler
	document.getElementById("mainTree").addEventListener("mousedown", function(event)
	{
		if (event.button != 2)
		{
			return;
		}
		popupShowing = true;
		var row = {},
			col = {},
			child = {};
		treeView.treebox.getCellAt(event.clientX, event.clientY, row, col, child);
		currentIndex = row.value;	
	}, true);
}