var selection = [];
var currentData = {};

function onSelect(event) {
	console.log("SELECT");
	var t = event.target;
	while(!t.hasAttribute('data-number'))
		t = t.parentNode;
	var number = t.getAttribute('data-number');
	if(event.ctrlKey) { // add to selection
		currentData.posts[number].selected = !currentData.posts[number].selected;
	} else if(event.shiftKey) { // expand selection
		var start = 0;
		var mind = currentData.posts.length;
		for(var i=0; i<currentData.posts.length; i++)
			if(currentData.posts[i].selected && (Math.abs(i-number) < mind))
			{
				mind = Math.abs(i-number);
				start = i;
			}
		var begin = Math.min(number, start);
		var end = Math.max(number, start);
		for(var i=begin; i<=end; i++)
			currentData.posts[i].selected = true;
	} else { // reset selection
		for(var i=0; i<currentData.posts.length; i++)
			currentData.posts[i].selected = i == number;
	}
	updateSelection();
}

function updateSelection() {
	var items = document.querySelectorAll('#container > div');
	//console.log(items);
	for(var i=0; i<currentData.posts.length; i++)
	{
		if(currentData.posts[i].selected)
			items[i].classList.add('rss-selected');
		else
			items[i].classList.remove('rss-selected');
	}
}

function onNewsClick(event) {
	var t = event.target;
	while(!t.hasAttribute('data-url'))
		t = t.parentNode;
	var url = t.getAttribute('data-url');
	t.classList.add('rss-read');
	window.parent.openSubpage(url, t.getAttribute('data-guid'));
}

function onDragStart(event) {
	var t = event.target;
	while(!t.hasAttribute('data-number'))
		t = t.parentNode;
	var number = t.getAttribute('data-number');
	if(!currentData.posts[number].selected) {
		for(var i=0; i<currentData.posts.length; i++)
			currentData.posts[i].selected = i == number;
	}
	var selection = [];
	for(var i=0; i<currentData.posts.length; i++)
	{
		if(currentData.posts[i].selected)
			selection.push(currentData.posts[i]);
	}
	if(selection.length == 1)
	{
		var url = selection[0].url;
		event.dataTransfer.setData('text/x-moz-url-data', url);
		event.dataTransfer.setData('text/x-moz-url-desc', selection[0].title);
		event.dataTransfer.setData("text/uri-list", url);
		event.dataTransfer.setData("text/x-tabsmoz-type", 'rsslink');
		event.dataTransfer.setData("text/x-tabsmoz-node", selection[0].guid);
		console.log("Started dragging url");
	} else {
		event.dataTransfer.setData('text/x-tabsmoz-nodelist', selection.map(function (val) {
			return val.guid;
		}));
		event.dataTransfer.setData('text/x-tabsmoz-nodetype', 'rsslink');
		console.log("Started dragging nodelist " + selection.length);
	}
}

function clearFeed() {
	var container = document.getElementById('container');
	while(container.firstChild)
	{
		container.removeChild(container.firstChild);
	}

}

function showFeed (data, type) {
	currentData = data;
	selection = [];
	var templates = [
		'row-template',
		'card-template',
		'magazine-template'
	];
	var container = document.getElementById('container');
	while(container.firstChild)
		container.removeChild(container.firstChild);
	container.classList.remove('rss-rows');
	container.classList.remove('rss-cards');
	container.classList.remove('rss-magazinen');
	switch(type) {
		case 0: container.classList.add('rss-rows'); break;
		case 1: container.classList.add('rss-cards'); break;
		case 2: container.classList.add('rss-magazinen'); break;
	}

	var template = document.getElementById(templates[type]);
	data.posts.sort(function(a, b) {
		if(a.pubDate < b.pubDate)
			return 1;
		if(a.pubDate > b.pubDate)
			return -1;
		return 0;
	});
	for(var i=0; i<data.posts.length; i++)
	{
		var item = template.cloneNode(true);
		item.removeAttribute('id');
		item.setAttribute('data-number', i); 
		var post = data.posts[i];
		item.querySelector('.rss-title').textContent = post.title;
		item.querySelector('.rss-content').textContent = post.content; //.setAttribute('value', post.content);
		item.querySelector('.rss-date').textContent = post.pubDate ? (new Date(post.pubDate)).toLocaleDateString() : "";
		item.querySelector('.rss-author').textContent = post.author || "";

		if(post.read)
			item.classList.add('rss-read');
		if(post.selected === undefined)
			post.selected = false;

		//var img = item.querySelector('.rss-content').querySelector('img');
		if(post.img && type > 0)
		{
			//console.log("[RSS] Image found");
			item.querySelector('.rss-image').src = post.img;
		}

		/*if(type == 0)
		{
			item.querySelector('.rss-content').innerHTML = item.querySelector('.rss-content').textContent;
		}*/

		item.setAttribute('data-url', post.url);
		item.setAttribute('data-guid', post.guid);
		item.addEventListener('dblclick', onNewsClick);
		item.addEventListener('mouseup', onSelect);
		item.addEventListener('dragstart', onDragStart);

		container.appendChild(item);
	}
}

document.addEventListener('keypress', function(event) {
	if (event.ctrlKey && event.which !== 0 && String.fromCharCode(event.which).toLowerCase() == 'a') 
	{
    	for(var i=0; i<currentData.posts.length; i++)
			currentData.posts[i].selected = true;
		updateSelection();
    }
});